
--[[//////////////////////////////////////////////////

    BANK'S WATCH

    This is a lightweight add-on to keep track of your
    bank, reagent, void storage, guild bank inventory.

    Since things can change often, the add-on will use
    collectgarbage() everytime it scans an update to
    avoid increase in memory usage.

    The add-on will load and work directly with saved
    file rather than loading the database into memory.

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]


BW_FRAMEWHITESPACE = 4
BW_BUTTON_HEIGHT = 32
BW_ROW = 12
BW_NUM_REAGENT_SLOTS = 98
BW_NUM_VOID_SLOTS = VOID_STORAGE_MAX or 80
BW_REAGENT_PAGES = 1
BW_VOID_PAGES = VOID_STORAGE_PAGES or 2
MAX_GUILDBANK_SLOTS_PER_TAB = MAX_GUILDBANK_SLOTS_PER_TAB or 98
BW_ACTIVE_TABLE = {}
BW_IsBankOpen = false

BW = {['items'] = {}, ['lookup'] = {}, ['lookuprequest'] = false} -- Contains search

BWMSG_C = {
    ['ERROR'] = "|cffFF00E1",
    ['INFO'] = "|cff35FCB5",--"|cffB5EDFF",
    ['EVENT'] = "|cffFFBF00",--"|cff35FCB5",
    ['DEBUG'] = "|cffE0FC35",
    ['DEFAULT'] = "|cff30DFEF",
    ['BOOL'] = "|cff2BED48",
    ['WARN'] = "|cffDBD3AF"
}

BW_LABEL = {
    ['bank'] = BWL[10001],
    ['reagent'] = BWL[10002],
    ['void'] = BWL[10003],
    ['guild'] = BWL[10004]
}

BW_OPT_HIDDEN = {
    ['iconsize'] = true,
    ['iconsizeslider'] = true,
    ['iconrow'] = true,
    ['iconrowslider'] = true,
    ['minimize'] = true
}


--[[//////////////////////////////////////////////////

    FUNCTIONS TRIGGERED VIA XML bankswatch.xml

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]

    function BW_OnLoad(self)

        ----- REGISTER FOR EVENTS
            self:RegisterEvent("BANKFRAME_OPENED")
            self:RegisterEvent("BANKFRAME_CLOSED")
            self:RegisterEvent("PLAYERBANKSLOTS_CHANGED")
            self:RegisterEvent("PLAYERBANKBAGSLOTS_CHANGED")
            self:RegisterEvent("PLAYERREAGENTBANKSLOTS_CHANGED")
            self:RegisterEvent("VOID_STORAGE_OPEN")
            self:RegisterEvent("VOID_STORAGE_UPDATE")
            self:RegisterEvent("VOID_STORAGE_CONTENTS_UPDATE")
            self:RegisterEvent("VOID_TRANSFER_DONE")
            self:RegisterEvent("GUILDBANKFRAME_OPENED")
            self:RegisterEvent("GUILDBANKBAGSLOTS_CHANGED")
            self:RegisterEvent("GUILDBANK_UPDATE_TABS")
            --self:RegisterEvent("ADDON_LOADED")
            self:RegisterEvent("PLAYER_ENTERING_WORLD")
            self:RegisterEvent("VARIABLES_LOADED")
            self:RegisterEvent("PLAYER_REGEN_DISABLED")
            self:RegisterEvent("GET_ITEM_INFO_RECEIVED")

        DEFAULT_CHAT_FRAME:AddMessage(BWMSG_C.DEFAULT..BWL[10000])

        ------ SLASH COMMANDS
            SLASH_BW1 = "/BW"
            SLASH_BW2 = "/bw"
            SLASH_BW3 = "/Bw"
            SLASH_BW4 = "/bW"
            SLASH_BW5 = "/Bankswatch"
            SLASH_BW6 = "/BanksWatch"
            SLASH_BW7 = "/BANKSWATCH"
            SLASH_BW8 = "/bankswatch"

            SlashCmdList["BW"] = BW_Main

            BW_CreateFrames()
            table.insert(UISpecialFrames, BW.mainframe.bank:GetName())
            table.insert(UISpecialFrames, BW.mainframe.reagent:GetName())
            table.insert(UISpecialFrames, BW.mainframe.void:GetName())
            table.insert(UISpecialFrames, BW.mainframe.guild:GetName())
            table.insert(UISpecialFrames, BW.mainframe.search:GetName())
    end

    function BW_OnEvent(self, event, ...)
        local items_available = newtable()

        --BWprint(event)
        if event == "VARIABLES_LOADED" then
            BWprint(BWMSG_C.EVENT.."Variables loaded. Initializing.", 2)
            BWprint(BWMSG_C.INFO.."Running version: "..GetAddOnMetadata("BanksWatch", "Version"), 1)

            if BWsavedtable then
                BW_BUTTON_HEIGHT = BWsavedtable['iconsize'] or 32
                BW_ROW = BWsavedtable['iconrow'] or 12

                if BWsavedtable['minimize'] then BW.mainframe.togglebutton:Show() end
            end

            BWI.Main.Bank.Buttons()
            BWI.Main.Reagent.Buttons()
            BWI.Main.Void.Buttons()
            BWI.Main.Guild.Buttons()

        elseif event == "GET_ITEM_INFO_RECEIVED" then
            local itemid = ...
            if BW.lookup[itemid] then BW.lookup[itemid] = nil end

            if BW.lookuprequest and not next(BW.lookup) then -- Try to minimize the amount of updates
                BW.lookuprequest = false
                BW_Update()
                BW_ReagentUpdate()
                BW_VoidUpdate()
                BW_GuildUpdate()
            end

        elseif event == "PLAYER_ENTERING_WORLD" then
            -- When loading, BAG_UPDATE triggers, but I only want when player modifies their bank bags
            self:RegisterEvent("PLAYER_REGEN_DISABLED")
            self:RegisterEvent("BAG_UPDATE")

            BW_Initialize()
            BW.Config.Create()

        elseif event == "PLAYERBANKSLOTS_CHANGED" then
            BW_Scan()

        elseif event == "BAG_UPDATE" then
            local bagid = ...

            if bagid == -1 or (bagid >= (NUM_BAG_SLOTS + 1) and bagid <= (NUM_BAG_SLOTS + NUM_BANKBAGSLOTS)) then
                -- bank bags
                BW_Scan()
            end

        elseif event == "PLAYERREAGENTBANKSLOTS_CHANGED" then
            BW_ReagentScan()

        elseif event == "VOID_STORAGE_OPEN" or event == "VOID_STORAGE_UPDATE" or event == "VOID_STORAGE_CONTENTS_UPDATE" or event == "VOID_TRANSFER_DONE" then
            BW_VoidScan()

        elseif event == "BANKFRAME_OPENED" then
            BW_IsBankOpen = true

            BWhideall()
            BW_Scan()
            BW_ReagentScan()

        elseif event == "BANKFRAME_CLOSED" then
            BW_IsBankOpen = false

        elseif event == "GUILDBANKFRAME_OPENED" or event == "GUILDBANK_UPDATE_TABS" then
            BW_RefreshGuildTab()

        elseif event == "GUILDBANKBAGSLOTS_CHANGED" then
            -- Guild can change outside the scope of the player. Refresh everytime the guild bank is opened.
            -- This will trigger when information about tabs is found.
            BW_GuildScan()
        elseif event == "PLAYER_REGEN_DISABLED" then
            -- Entering combat, hide any open bags
            BWhideall()
        end
    end

--[[//////////////////////////////////////////////////

    MAIN FUNCTIONS

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]

    function BW_Initialize()

        hooksecurefunc("ContainerFrameItemButton_OnModifiedClick", BW_ContainerFrameItemButton_OnModifiedClick)
        hooksecurefunc("ChatFrame_OnHyperlinkShow", BW_ChatFrame_OnHyperlinkShow)
        hooksecurefunc("ChatEdit_InsertLink", BW_InsertLink)

        local player = UnitName("player")
        local server = GetRealmName()
        local faction = UnitFactionGroup("player")

        if (player == nil) or (player == UNKNOWNOBJECT) or (player == UNKNOWNBEING) then
            return
        end

        if BWsavedtable and BWsavedtable[server] and BWsavedtable[server][player.."."..string.sub(faction, 1, 1)] then
            BW_LoadTable(server, player, faction)
        else
            BWprint(BWMSG_C.EVENT.."New server found")
            BW_template(server, player, faction)
        end
    end

    function BW_Main(input)
        -- this is called when we type /AS or clicks the AS tab
        BWprint(BWMSG_C.INFO.."That's a lot of stuff.", 1)
        if input then input = string.lower(input) end

        if input == "toggle" then
            if BW.mainframe:IsVisible() then
                BW.mainframe:Hide()
            else
                BW.mainframe:Show()
            end
        elseif input == "debug" then
            BWdebug = not BWdebug
            BWprint(BWMSG_C.INFO.."Debug:|r "..BWMSG_C.BOOL..tostring(BWdebug), 1)
        end
    end

    function BW_ContainerFrameItemButton_OnModifiedClick(self)

        if IsShiftKeyDown() then
            local bag, item = self:GetParent():GetID(), self:GetID()
            local link = GetContainerItemLink(bag, item)

            if BW.mainframe.searchbar:HasFocus() and BW.mainframe.searchbar:GetText() ~= text then

                BW.mainframe.searchbar:SetText(GetItemInfo(link))
            end
        end
    end

    function BW_ChatFrame_OnHyperlinkShow(self, link, text, button)

        if IsShiftKeyDown() and link then
            if string.find(link, 'achievement:*') or string.find(link,'spell:*') then
                return false
            end

            if BW.mainframe.searchbar:HasFocus() and BW.mainframe.searchbar:GetText() ~= text then
                BW.mainframe.searchbar:SetText(GetItemInfo(link))
            end
        end
    end

    function BW_InsertLink(text)

        if BW.mainframe.searchbar:HasFocus() and BW.mainframe.searchbar:GetText() ~= text then
            BW.mainframe.searchbar:SetText(string.find(text, "^|c%x+|H(.+)|h%[.*%]"))
        end
    end

    function BWdropDownMenuItem_OnClick(self)
        local server, player, faction
        local value = self.value

        server, player = value:match"([^.]*).(.*)"
        player, faction = player:match"([^.]*).(.*)"
        faction = faction == "H" and "Horde" or faction == "A" and "Alliance" or faction == "N" and "Neutral"

        BW_LoadTable(server, player, faction)
        BWdropDownMenuButton:Click()
    end

    function BW_ScrollbarUpdate()
        local x, i, current, name, bag, loc, num
        local banks = {"bank", "reagent", "void", "guild"}

        if not next(BW.items) then return end
        BW.sort = newtable()
        BW.search = newtable()

        for x = 1, #banks do BW.sort[banks[x]] = newtable() end
        -- sort results, group by location, stack same item
        for x = 1, #BW.items do
            name = BW.items[x].name
            loc, num = BW.items[x].loc:match"([^.]*).(.*)"
            num = tonumber(num)

            if not BW.sort[loc][num] then BW.sort[loc][num] = newtable() end
            if not BW.sort[loc][num][name] then
                BW.sort[loc][num][name] = newtable()
                temp = BW.sort[loc][num][name]
                temp['count'] = 0
                temp['link'] = BW.items[x].link
                temp['quality'] = BW.items[x].quality
                temp['icon'] = BW.items[x].icon
                temp['name'] = name
                temp['ilvl'] = BW.items[x].ivl
                temp['loc'] = BW.items[x].loc
            end
            BW.sort[loc][num][name]['stack'] = newtable()
            BW.sort[loc][num][name]['stack'][#BW.sort[loc][num][name]['stack'] + 1] = BW.items[x]
            BW.sort[loc][num][name]['count'] = BW.sort[loc][num][name]['count'] + BW.items[x].count
        end

        for x = 1, #banks do
            for i = 1, table.maxn(BW.sort[banks[x]]) do
                if BW.sort[banks[x]][i] then
                    for k, v in pairs(BW.sort[banks[x]][i]) do
                        BW.search[#BW.search + 1] = v
                    end
                end
            end
        end

        local BWnumberofitems = #BW.search
        local currentscrollbarvalue = FauxScrollFrame_GetOffset(BW.mainframe.search.scroll)

        FauxScrollFrame_Update(BW.mainframe.search.scroll, BWnumberofitems, BWrowsthatcanfit(), BW_BUTTON_HEIGHT)

        local r, g, b, idx, link, hexcolor, itemRarity
        
        for x = 1, BWrowsthatcanfit() do
            -- Get the appropriate item, which will be x + value
            idx = x + currentscrollbarvalue

            BW_ResetButton(BW.mainframe.search.list.buttons[x])
            BW.mainframe.search.list.buttons[x]:SetBackdropColor(0, 0, 0, 0.7)
            BW.mainframe.search.list.buttons[x].leftstring:SetText("")
            BW.mainframe.search.list.buttons[x]:Hide()
            BW.mainframe.search.enddot:Hide()

            if BW.search[idx] then
                r, g, b, hexcolor = GetItemQualityColor(BW.search[idx].quality)

                BW.mainframe.search.list.buttons[x].quality = BW.search[idx].quality
                BW.mainframe.search.list.buttons[x].icon:SetTexture(BW.search[idx].icon)
                BW.mainframe.search.list.buttons[x].leftstring:SetText("|c"..hexcolor..BW.search[idx].name)

                if BW.search[idx].quality > 1 then
                    BW.mainframe.search.list.buttons[x]:SetBackdropBorderColor(r, g, b, 0.5)
                end
                if BW.search[idx].count > 1 then
                    BW.mainframe.search.list.buttons[x].text:SetText(BW.search[idx].count)
                end
                if BW.search[idx].ilvl and BW.search[idx].ilvl ~= "" then
                    BW.mainframe.search.list.buttons[x].ilvl:SetText(BW.search[idx].ilvl)
                    BW.mainframe.search.list.buttons[x].ilvl:SetTextColor(r, g, b, 1)
                end

                loc, num = BW.search[idx].loc:match"([^.]*).(.*)"
                location = tonumber(num) == 1 and BW_LABEL[loc] or BW_LABEL[loc].." "..num
                BW.mainframe.search.list.buttons[x].location = {['loc'] = loc, ['num'] = num}
                BW.mainframe.search.list.buttons[x].rightstring:SetText(location)
                BW.mainframe.search.list.buttons[x].link = BW.search[idx].link
                BW.mainframe.search.list.buttons[x]:Show()
                BW.mainframe.search.enddot:Show()
                if idx == BWnumberofitems then
                    BW.mainframe.search.enddot:Hide()
                end
            end
        end
    end

--[[//////////////////////////////////////////////////

    SECONDARY FUNCTIONS

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]


    function BW_Scan()

        if not BW_IsBankOpen then return end -- Circumvent early BAG_UPDATE
        local x, counter, count, link, bag, slot, slots
        local bank = BW_GetPlayerBank('bank', true)

        if not bank[1] then bank[1] = newtable() end
        for x = 1, NUM_BANKGENERIC_SLOTS do
            _, count, _, _, _, _, link = GetContainerItemInfo(BANK_CONTAINER, x)

            if link then
                bank[1][x] = newtable()
                bank[1][x]['count'] = count
                bank[1][x]['link'] = link
            else
                bank[1][x] = deltable(bank[1][x])
            end
        end

        for bag = ITEM_INVENTORY_BANK_BAG_OFFSET + 1, ITEM_INVENTORY_BANK_BAG_OFFSET + NUM_BANKBAGSLOTS do
            if not bank[bag] then bank[bag] = newtable() end
            slots = GetContainerNumSlots(bag)
            bank[bag]['slots'] = slots

            for x = 1, slots do
                _, count, _, _, _, _, link = GetContainerItemInfo(bag, x)

                if link then
                    bank[bag][x] = newtable()
                    bank[bag][x]['count'] = count
                    bank[bag][x]['link'] = link
                else
                    bank[bag][x] = deltable(bank[bag][x])
                end
            end
        end

        bank = nil
        collectgarbage()

        BW_Update()
    end

    function BW_ReagentScan()

        local x,  page, count, link
        local reagent = BW_GetPlayerBank('reagent', true)

        for page = 1, BW_REAGENT_PAGES do
            if not reagent[page] then reagent[page] = newtable() end

            for x = 1, BW_NUM_REAGENT_SLOTS do
                _, count, _, _, _, _, link = GetContainerItemInfo(REAGENTBANK_CONTAINER, x)

                if link then
                    reagent[page][x] = newtable()
                    reagent[page][x]['count'] = count
                    reagent[page][x]['link'] = link
                else
                    reagent[page][x] = deltable(reagent[page][x])
                end
            end
        end

        reagent = nil
        collectgarbage()

        BW_ReagentUpdate()
    end

    function BW_VoidScan()

        local x, page, itemID
        local void = BW_GetPlayerBank('void', true)

        for page = 1, BW_VOID_PAGES do
            if not void[page] then void[page] = newtable() end

            for x = 1, BW_NUM_VOID_SLOTS do
                itemID = GetVoidItemInfo(page, x)

                if itemID then
                    void[page][x] = newtable()
                    void[page][x]['link'] = page == 1 and GetVoidItemHyperlinkString(x) or GetVoidItemHyperlinkString(BW_NUM_VOID_SLOTS + x)
                    void[page][x]['count'] = 1
                else
                    void[page][x] = deltable(void[page][x])
                end
            end
        end

        void = nil
        collectgarbage()

        BW_VoidUpdate()
    end

    function BW_GuildScan()

        local x, count, item
        local guild = BW_GetPlayerBank('guild', true)
        local page = GetCurrentGuildBankTab()

        if guild[page].access then
            for x = 1, MAX_GUILDBANK_SLOTS_PER_TAB do

                item = GetGuildBankItemLink(page, x)
                _, count = GetGuildBankItemInfo(page, x)

                if item then
                    guild[page][x] = newtable()
                    guild[page][x]['link'] = item
                    guild[page][x]['count'] = count
                else
                    guild[page][x] = deltable(guild[page][x])
                end
            end
        end

        guild = nil
        collectgarbage()

        BW_GuildUpdate()
    end


    function BW_Update()

        local page, total, counter
        local bank = BW_GetPlayerBank('bank')

        total = 0
        for page = 1, #bank do
            total = total + (bank[page] and bank[page]['slots'] or 0)
        end
        BW_ValidateBankBags(total)

        counter = 0
        for page = 1, #bank do
            if bank[page] then

                BW_UpdatePage(BW.mainframe.bank, bank[page]['slots'], bank[page], counter)
                counter = counter + bank[page]['slots']
            end
        end
    end

    function BW_ReagentUpdate()

        local page
        local reagent = BW_GetPlayerBank('reagent')

        for page = 1, BW_REAGENT_PAGES do
            BW_UpdatePage(BW.mainframe.reagent[page], BW_NUM_REAGENT_SLOTS, reagent[page])
        end
    end

    function BW_VoidUpdate()

        local page
        local void = BW_GetPlayerBank('void')

        for page = 1, BW_VOID_PAGES do
            BW_UpdatePage(BW.mainframe.void[page], BW_NUM_VOID_SLOTS, void[page])
        end
    end

    function BW_GuildUpdate()

        local page
        local guild = BW_GetPlayerBank('guild')

        for page = 1, MAX_GUILDBANK_TABS do
            if guild[page] then

                if guild[page].icon then
                    BW.mainframe.guild[page].tab:SetNormalTexture(guild[page].icon)
                    BW.mainframe.guild[page].tab.name = guild[page].name
                    BW.mainframe.guild[page].tab:Show()
                else
                    BW.mainframe.guild[page].tab:Hide()
                end

                if guild[page].access then -- Validate user has access to view tab
                    BW.mainframe.guild[page].tab:Enable()
                    BW.mainframe.guild[page].tab:GetNormalTexture():SetDesaturated(false)
                    BW_UpdatePage(BW.mainframe.guild[page], MAX_GUILDBANK_SLOTS_PER_TAB, guild[page])
                else
                    BW.mainframe.guild[page].tab:Disable()
                    BW.mainframe.guild[page].tab:GetNormalTexture():SetDesaturated(true)
                end
            else
                BW.mainframe.guild[page].tab:Hide()
            end
        end
    end

    function BW_RefreshGuildTab()

        local x, name, icon, isViewable
        local guild = BW_GetPlayerBank('guild')
        local tabs = GetNumGuildBankTabs()

        for x = 1, MAX_GUILDBANK_TABS do
            if x <= tabs then
                if not guild[x] then guild[x] = newtable() end
                name, icon, isViewable = GetGuildBankTabInfo(x)
                guild[x]['icon'] = icon
                guild[x]['name'] = name
                guild[x]['access'] = isViewable

                BW.mainframe.guild[x].tab:SetNormalTexture(icon)
                BW.mainframe.guild[x].tab.name = name
                BW.mainframe.guild[x].tab:Show()
            else
                guild[x] = deltable(guild[x])
                BW.mainframe.guild[x].tab:Hide()
            end
        end
    end


    function BW_UpdatePage(page, maxslots, data, optstart)
        local x, item
        optstart = optstart or 0

        for x = 1, maxslots do

            BW_ResetButton(page.buttons[x + optstart])
            if data and data[x] then

                item = {GetItemInfo(data[x].link)}
                if not next(item) then BW.lookuprequest = true; BW.lookup[tonumber(BW_GetItemIdFromLink(data[x].link))] = true end
                BW_UpdateButton(page.buttons[x + optstart], item, data[x])
            end
        end
    end

    function BW_UpdateButton(button, item, data)
        local r, g, b, ilvl
        item[3] = item[3] or 1
        r, g, b = GetItemQualityColor(item[3])

        button.name = item[1]
        button.count = data.count
        button.quality = item[3]
        button.link = item[2]

        button.icon:SetTexture(item[10])
        button.icon:SetTexCoord(0.1, 0.9, 0.1, 0.9)
        
        if data.count > 1 then
            button.text:SetText(data.count)
        end

        if item[3] > 1 then
            button:SetBackdropBorderColor(r, g, b, 1)
        end

        if item[6] == BATTLE_PET_SOURCE_2 then -- QUEST
            button:SetBackdropBorderColor(1, 1, 0, 1)
            button.quest = true

        elseif item[6] == ARMOR or item[6] == ENCHSLOT_WEAPON then -- ILVL
            if item[4] == 1 then
                ilvl = BW_Detectilvl(item[2])
                if ilvl then
                    button.ilvl:SetText(ilvl)
                    button.ilvl:SetTextColor(r, g, b, 1)
                end
            else
                button.ilvl:SetText(item[4])
                button.ilvl:SetTextColor(r, g, b, 1)
            end
        end
    end

    function BW_ResetButton(button)

        button.icon:SetTexture("")
        button:SetBackdropBorderColor(0, 0, 0, 1)
        button.link = nil
        button.quality = nil
        button.quest = nil
        button.name = nil
        button.count = nil
        button.text:SetText("")
        button.ilvl:SetText("")
    end


    function BW_Search(text)

        BW.items = newtable()

        BW_BankSearch(text)
        BW_ReagentSearch(text)
        BW_VoidSearch(text)
        BW_GuildSearch(text)

        BW_ScrollbarUpdate()
    end

    function BW_BankSearch(text)

        local x, page, counter
        local bank = BW_GetPlayerBank('bank')
        if not bank or next(bank) == nil then
            BWprint(BWMSG_C.ERROR.."Bank is empty")
            return false
        end

        x = 1
        for page = 1, #bank do
            if bank[page] then
                for counter = 1, bank[page]['slots'] do

                    name = BW.mainframe.bank.buttons[x].name
                    if name and text then
                        if BW_find(name, text) then
                            BW.mainframe.bank.buttons[x].search:Hide()
                            BW.mainframe.countbutton.count = BW.mainframe.countbutton.count + BW.mainframe.bank.buttons[x].count
                            BW.items[#BW.items + 1] = newtable()
                            BW.items[#BW.items]['link'] = BW.mainframe.bank.buttons[x].link
                            BW.items[#BW.items]['quality'] = BW.mainframe.bank.buttons[x].quality
                            BW.items[#BW.items]['name'] = BW.mainframe.bank.buttons[x].name
                            BW.items[#BW.items]['count'] = BW.mainframe.bank.buttons[x].count
                            BW.items[#BW.items]['quest'] = BW.mainframe.bank.buttons[x].quest
                            BW.items[#BW.items]['icon'] = BW.mainframe.bank.buttons[x].icon:GetTexture()
                            BW.items[#BW.items]['ilvl'] = BW.mainframe.bank.buttons[x].ilvl:GetText()
                            BW.items[#BW.items]['loc'] = "bank."..(page == 1 and page or page - 4)
                        else
                            BW.mainframe.bank.buttons[x].search:Show()
                        end
                    else
                        BW.mainframe.bank.buttons[x].search:Hide()
                    end
                    x = x + 1
                end
            end
        end
    end

    function BW_ReagentSearch(text)

        local x, page
        local reagent = BW_GetPlayerBank('reagent')
        if not reagent or next(reagent) == nil then
            BWprint(BWMSG_C.ERROR.."Reagent bank is empty")
            return false
        end

        for page = 1, BW_REAGENT_PAGES do
            for x = 1, BW_NUM_REAGENT_SLOTS do
                name = BW.mainframe.reagent[page].buttons[x].name
                if name and text then
                    if BW_find(name, text) then
                        BW.mainframe.reagent[page].buttons[x].search:Hide()
                        BW.mainframe.countbutton.count = BW.mainframe.countbutton.count + BW.mainframe.reagent[page].buttons[x].count
                        BW.items[#BW.items + 1] = newtable()
                        BW.items[#BW.items]['link'] = BW.mainframe.reagent[page].buttons[x].link
                        BW.items[#BW.items]['quality'] = BW.mainframe.reagent[page].buttons[x].quality
                        BW.items[#BW.items]['name'] = BW.mainframe.reagent[page].buttons[x].name
                        BW.items[#BW.items]['count'] = BW.mainframe.reagent[page].buttons[x].count
                        BW.items[#BW.items]['quest'] = BW.mainframe.reagent[page].buttons[x].quest
                        BW.items[#BW.items]['ilvl'] = BW.mainframe.reagent[page].buttons[x].ilvl:GetText()
                        BW.items[#BW.items]['icon'] = BW.mainframe.reagent[page].buttons[x].icon:GetTexture()
                        BW.items[#BW.items]['loc'] = "reagent."..page
                    else
                        BW.mainframe.reagent[page].buttons[x].search:Show()
                    end
                else
                    BW.mainframe.reagent[page].buttons[x].search:Hide()
                end
            end
        end
    end

    function BW_VoidSearch(text)

        local x, proceed, page
        local void = BW_GetPlayerBank('void')
        
        proceed = false
        for page = 1, BW_VOID_PAGES do
            if void[page] and next(void[page]) then
                proceed = true
                break
            end
        end
        if not proceed then
            BWprint(BWMSG_C.ERROR.."Void storage is empty")
            return false
        end

        for page = 1, BW_VOID_PAGES do
            for x = 1, BW_NUM_VOID_SLOTS do
                name = BW.mainframe.void[page].buttons[x].name
                if name and text then
                    if BW_find(name, text) then
                        BW.mainframe.void[page].buttons[x].search:Hide()
                        BW.mainframe.countbutton.count = BW.mainframe.countbutton.count + BW.mainframe.void[page].buttons[x].count
                        BW.items[#BW.items + 1] = newtable()
                        BW.items[#BW.items]['link'] = BW.mainframe.void[page].buttons[x].link
                        BW.items[#BW.items]['quality'] = BW.mainframe.void[page].buttons[x].quality
                        BW.items[#BW.items]['name'] = BW.mainframe.void[page].buttons[x].name
                        BW.items[#BW.items]['count'] = BW.mainframe.void[page].buttons[x].count
                        BW.items[#BW.items]['quest'] = BW.mainframe.void[page].buttons[x].quest
                        BW.items[#BW.items]['ilvl'] = BW.mainframe.void[page].buttons[x].ilvl:GetText()
                        BW.items[#BW.items]['icon'] = BW.mainframe.void[page].buttons[x].icon:GetTexture()
                        BW.items[#BW.items]['loc'] = "void."..page
                    else
                        BW.mainframe.void[page].buttons[x].search:Show()
                    end
                else
                    BW.mainframe.void[page].buttons[x].search:Hide()
                end
            end
        end
    end

    function BW_GuildSearch(text)

        local x, proceed, page
        local guild = BW_GetPlayerBank('guild')

        proceed = false
        for page = 1, MAX_GUILDBANK_TABS do
            if guild[page] and next(guild[page]) then
                proceed = true
                break
            end
        end
        if not proceed then
            BWprint(BWMSG_C.ERROR.."Guild bank is empty")
            return false
        end

        for page = 1, MAX_GUILDBANK_TABS do

            if guild[page] and guild[page].access then
                for x = 1, MAX_GUILDBANK_SLOTS_PER_TAB do
                    name = BW.mainframe.guild[page].buttons[x].name
                    if name and text then
                        if BW_find(name, text) then
                            BW.mainframe.guild[page].buttons[x].search:Hide()
                            BW.mainframe.countbutton.count = BW.mainframe.countbutton.count + BW.mainframe.guild[page].buttons[x].count
                            BW.items[#BW.items + 1] = newtable()
                            BW.items[#BW.items]['link'] = BW.mainframe.guild[page].buttons[x].link
                            BW.items[#BW.items]['quality'] = BW.mainframe.guild[page].buttons[x].quality
                            BW.items[#BW.items]['name'] = BW.mainframe.guild[page].buttons[x].name
                            BW.items[#BW.items]['count'] = BW.mainframe.guild[page].buttons[x].count
                            BW.items[#BW.items]['quest'] = BW.mainframe.guild[page].buttons[x].quest
                            BW.items[#BW.items]['ilvl'] = BW.mainframe.guild[page].buttons[x].ilvl:GetText()
                            BW.items[#BW.items]['icon'] = BW.mainframe.guild[page].buttons[x].icon:GetTexture()
                            BW.items[#BW.items]['loc'] = "guild."..page
                        else
                            BW.mainframe.guild[page].buttons[x].search:Show()
                        end
                    else
                        BW.mainframe.guild[page].buttons[x].search:Hide()
                    end
                end
            end
        end
    end
