
--[[

    HELPER FUNCTIONS

    Anything that needs repetitive actions or calculations
    Tips: 
        -- Multiplication are faster than division
        -- Local is faster than global. Re-assign global as a new local if used a lot

]]

------ MAIN TABLE
    
    -- Table Pool for recycling tables
    local tables = setmetatable({}, {__mode = "kv"}) -- Weak table

    -- Get a new table
    function newtable()
        local t = next(tables) or {}
        tables[t] = nil
        return t
    end

    -- Delete table and add to table pool
    function deltable(t)
        if type(t) == "table" then
            for k, v in pairs(t) do
                if type(v) == "table" then
                    deltable(v)
                end
                t[k] = nil
            end
            
            setmetatable(t, nil)
            t[true] = true
            t[true] = nil
            tables[t] = true
        end

        return nil
    end

    function BW_template(server, player, faction)

        BWprint(BWMSG_C.EVENT.."Generating template: "..player)
        BW_ACTIVE_TABLE = {['server'] = server, ['player'] = player, ['faction'] = faction}

        local template = {['bank'] = {[1] = {['slots'] = NUM_BANKGENERIC_SLOTS}}, ['reagent'] = {},
                          ['void'] = {}, ['guild'] = {}}
        
        if not BWsavedtable then BWsavedtable = {} end
        BWsavedtable['iconsizeslider'] = 32
        BWsavedtable['iconsize'] = 32
        BWsavedtable['iconrowslider'] = 12
        BWsavedtable['iconrow'] = 12
        BWsavedtable['minimize'] = false

        if not BWsavedtable[server] then BWsavedtable[server] = {} end

        BWsavedtable[server][player.."."..string.sub(faction, 1, 1)] = template
    end

    function BW_LoadTable(name, player, faction)

        BW_ACTIVE_TABLE = {['server'] = name, ['player'] = player, ['faction'] = faction}
        BW.items = newtable()

        BW_Update()
        BW_ReagentUpdate()
        BW_VoidUpdate()
        BW_GuildUpdate()

        local text = BW.mainframe.searchbar:GetText()
        if text then
            BW.mainframe.searchbar:SetText("")
            BW.mainframe.searchbar:SetText(text)
        end
    end

    function BW_GetPlayerBank(bank, current)

        if not bank then return end
        if current then
            local player = UnitName("player")
            local server = GetRealmName()
            local faction = UnitFactionGroup("player")
            return BWsavedtable[server][player.."."..string.sub(faction, 1, 1)][bank]
        end

        return BWsavedtable[BW_ACTIVE_TABLE.server][BW_ACTIVE_TABLE.player.."."..string.sub(BW_ACTIVE_TABLE.faction, 1, 1)][bank]
    end

------ LOGGING
    local BWprintstack = -1
    local messagestring = ""

    function BWprint(message, level)

        if BWdebug or (level == 1) then

            if BWprintstack == -1 then
                messagestring = BWMSG_C.DEFAULT.."Bank's Watch|r: "
            end

            if not message then
                DEFAULT_CHAT_FRAME:AddMessage(messagestring..tostring(message))
                return false 
            end

            BWprintstack = BWprintstack + 1

            if (type(message) == "table") then
                local filler=string.rep(". . ", BWprintstack)

                DEFAULT_CHAT_FRAME:AddMessage(messagestring.."{table} --> ")
                for k, v in pairs(message) do
                    messagestring = filler.."["..k.."] = "
                    BWprint(v)
                end
            
            elseif (type(message) == "userdata") then

            elseif (type(message) == "function") then
                DEFAULT_CHAT_FRAME:AddMessage(messagestring.." A function(self)")
            elseif (type(message) == "boolean") then
                DEFAULT_CHAT_FRAME:AddMessage(messagestring..tostring(message))
            else
                DEFAULT_CHAT_FRAME:AddMessage(messagestring..tostring(message))
            end
            
            BWprintstack = BWprintstack - 1
       end
    end

    function BW_find(name, text)

        if name and string.find(string.lower(string.gsub(name, "[%-]", " ")), string.lower(string.gsub(text, "[%-]", " "))) then
            return true
        end
        return false
    end

------ VISIBILITY, CURSOR

    function GetCursorScaledPosition()
        local scale, x, y = UIParent:GetScale(), GetCursorPosition()
        return x / scale, y / scale
    end

    function BW_SetFrameLocation()
        local x, y = GetCursorScaledPosition()
        -- Only hide when cursor exits at the top or bottom of the option menu
        if (x < (GetScreenWidth()/2)) and (y > (GetScreenHeight()/2)) then
            if BW.mainframe.layout ~= 1 then BW_AnchorBagsTopLeft() end
        elseif (x < (GetScreenWidth()/2)) and (y < (GetScreenHeight()/2)) then
            if BW.mainframe.layout ~= 2 then BW_AnchorBagsBottomLeft() end
        elseif (x > (GetScreenWidth()/2)) and (y > (GetScreenHeight()/2)) then
            if BW.mainframe.layout ~= 3 then BW_AnchorBagsTopRight() end
        elseif (x > (GetScreenWidth()/2)) and (y < (GetScreenHeight()/2)) then
            if BW.mainframe.layout ~= 4 then BW_AnchorBagsBottomRight() end
        end
    end

    function BWrowsthatcanfit()

        local ourheight = math.ceil(BW.mainframe.search.list:GetHeight()) - BW_FRAMEWHITESPACE
        return math.floor(ourheight / BW_BUTTON_HEIGHT)
    end

function BWhideall()

    BW.mainframe.bank:Hide()
    BW.mainframe.reagent:Hide()
    BW.mainframe.void:Hide()
    BW.mainframe.guild:Hide()
end

local ilvlpattern = _G["ITEM_LEVEL"]:gsub("%%d", "(%%d+)")
local faketooltip = CreateFrame("GameTooltip", "BW_faketooltip", nil, "GameTooltipTemplate")
faketooltip:SetOwner(UIParent, "ANCHOR_NONE")

function BW_Detectilvl(itemlink)

    local text, ilvl

    faketooltip:SetHyperlink(itemlink)
    text = BW_faketooltipTextLeft2:GetText()
    if text then ilvl = tonumber(text:match(ilvlpattern)) end

    faketooltip:ClearLines()

    return ilvl or nil
end

function BW_GetItemIdFromLink(itemlink)

    local itemString = string.match(itemlink, "item[%-?%d:]+")
    local _, itemId = strsplit(":", itemString)
    return itemId
end
